# -*- coding: utf-8 -*-
"""The data model for registered tool functions in AgentScope."""
from copy import deepcopy
from dataclasses import field, dataclass
from typing import Callable, Literal, Type

from pydantic import BaseModel

from ._response import ToolResponse
from .._utils._common import _remove_title_field
from ..message import ToolUseBlock
from ..types import ToolFunction, JSONSerializableObject


@dataclass
class RegisteredToolFunction:
    """The registered tool function class."""

    name: str
    """The name of the tool function."""
    group: str | Literal["basic"]
    """The belonging group of the tool function"""
    source: Literal["function", "mcp_server", "function_group"]
    """"The type of the tool function, can be `function` or `mcp_server`."""
    original_func: ToolFunction
    """The original function"""
    json_schema: dict
    """The JSON schema of the tool function, which is used to validate the """
    preset_kwargs: dict[str, JSONSerializableObject] = field(
        default_factory=dict,
    )
    """The preset keyword arguments, which won't be presented in the JSON
    schema and exposed to the user."""
    extended_model: Type[BaseModel] | None = None
    """The base model used to extend the JSON schema of the original tool
    function, so that we can dynamically adjust the tool function."""
    mcp_name: str | None = None
    """The name of the MCP, if the tool function comes from an MCP server."""
    postprocess_func: Callable[
        [ToolUseBlock, ToolResponse],
        ToolResponse | None,
    ] | None = None
    """The post-processing function that will be called after the tool
    function is executed, taking the tool call block and tool
    response as arguments. If it returns `None`, the tool result will be
    returned as is. If it returns a `ToolResponse`, the returned block
    will be used as the final tool response."""

    @property
    def extended_json_schema(self) -> dict:
        """Get the JSON schema of the tool function, if an extended model is
        set, the merged JSON schema will be returned."""
        if self.extended_model is None:
            return self.json_schema

        # Merge the extended model with the original JSON schema
        extended_schema = self.extended_model.model_json_schema()

        merged_schema = deepcopy(self.json_schema)

        _remove_title_field(  # pylint: disable=protected-access
            extended_schema,
        )

        for key, value in extended_schema["properties"].items():
            if key in self.json_schema["function"]["parameters"]["properties"]:
                raise ValueError(
                    f"The field `{key}` already exists in the original "
                    f"function schema of `{self.name}`. Try to use a "
                    "different name.",
                )

            merged_schema["function"]["parameters"]["properties"][key] = value

            if key in extended_schema.get("required", []):
                if "required" not in merged_schema["function"]["parameters"]:
                    merged_schema["function"]["parameters"]["required"] = []
                merged_schema["function"]["parameters"]["required"].append(key)
        return merged_schema



# -*- coding: utf-8 -*-
"""Setup script for AgentScope."""
import re
import setuptools

# obtain version from src/agentscope/_version.py
with open("src/agentscope/_version.py", encoding="UTF-8") as f:
    VERSION = re.search(
        r'^__version__\s*=\s*[\'"]([^\'"]*)[\'"]',
        f.read(),
        re.MULTILINE,
    ).group(1)

NAME = "agentscope"
URL = "https://github.com/agentscope-ai/agentscope"

minimal_requires = [
    "aioitertools",
    "anthropic",
    "dashscope",
    "docstring_parser",
    "json5",
    "json_repair",
    "mcp>=1.13",
    "numpy",
    "openai",
    "python-datauri",
    "opentelemetry-api",
    "opentelemetry-sdk",
    "opentelemetry-exporter-otlp",
    "python-socketio",
    "shortuuid",
    "tiktoken",
    "sounddevice",
]

extra_requires = [
    # Model
    "ollama",
    "google-genai",
    "Pillow",
    "transformers",
    "jinja2",
    # Evaluator
    "ray",
    # Long-term memory
    "mem0ai",
    "packaging",
    # RAG
    "pypdf",
    "nltk",
    "qdrant-client",
]

dev_requires = [
    "pre-commit",
    "pytest",
    "sphinx-gallery",
    "furo",
    "myst_parser",
    "matplotlib",
]

with open("README.md", "r", encoding="UTF-8") as fh:
    long_description = fh.read()

setuptools.setup(
    name=NAME,
    version=VERSION,
    author="SysML team of Alibaba Tongyi Lab ",
    author_email="gaodawei.gdw@alibaba-inc.com",
    description="AgentScope: A Flexible yet Robust Multi-Agent Platform.",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url=URL,
    download_url=f"{URL}/archive/v{VERSION}.tar.gz",
    keywords=["deep-learning", "multi agents", "agents"],
    package_dir={"": "src"},
    packages=setuptools.find_packages("src"),
    install_requires=minimal_requires,
    extras_require={
        "full": minimal_requires + extra_requires,
        "dev": minimal_requires + extra_requires + dev_requires,
    },
    license="Apache-2.0",
    classifiers=[
        "Development Status :: 4 - Beta",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.10",
        "Operating System :: OS Independent",
    ],
    python_requires=">=3.10",
    entry_points={},
)
